/*
 * @brief emWin Hello World example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "GUI.h"
#include "board.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

/* LCD width and height (from board LCD setup descriptor) */
#define LCD_WIDTH       BOARD_LCD.PPL
#define LCD_HEIGHT      BOARD_LCD.LPP

/* Frame buffer address for lcd */
#define FRAMEBUFFER_ADDR        0xA0800000

/* ****************************************** remove *************************************************/
#include <stddef.h>
#include <string.h>

#include "GUI_Private.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
//
// Minimum size of a data block. Should be a potency of two
//
#ifndef   GUI_ALLOC_MIN_BLOCKSIZE
  #define GUI_ALLOC_MIN_BLOCKSIZE 16UL
#endif

#ifndef   GUI_MAX_REQUEST_PERCENT
  #define GUI_MAX_REQUEST_PERCENT 90
#endif

//
// States of block usage
//
#define BLOCK_FREE      0    // Block is free
#define BLOCK_ALLOCATED 1    // Block is allocated but not locked
#define BLOCK_LOCKED    2    // Block is allocated and locked. Values above means block is multiple locked

#define FILLBYTE        0xCC // WIN32 build fills up freed memory with this value
#define NUM_BLOCKS_INIT 2    // Number of blocks in first step of initialization

//
// Returns the size of the payload area of a data block
//
#define NET(x)   (U32)((U32)x - (U32)sizeof(MEM_TAG) - (U32)sizeof(U32))

//
// Returns the size of a data block:
// - including MEM_TAG structure (at the beginning)
// - and size marker (at the end of a block)
//
#define GROSS(x) (U32)((U32)x + (U32)sizeof(MEM_TAG) + (U32)sizeof(U32))

/*********************************************************************
*
*       Types
*
**********************************************************************
*/
/*********************************************************************
*
*       MEM_TAG
*
*  Function description
*    Memory information structure.
*/
typedef struct {
  U32 Block; // Index of block which refers to this memory block
  U32 Size;  // Gross size of memory block
} MEM_TAG;

/*********************************************************************
*
*       FREE_TAG
*
*  Function description
*    Used to link free blocks of memory. Locations of these tags are the
*    unused memory areas of free blocks.
*/
typedef struct FREE_TAG FREE_TAG;
typedef struct TAG      TAG;

struct FREE_TAG {
  FREE_TAG * pNext; // Pointer to next free tag
  FREE_TAG * pPrev; // Pointer to previous free tag
  TAG      * pTag;  // Pointer to memory tag
};

/*********************************************************************
*
*       TAG
*
*  Function description
*    Complete memory block:
*    - In case of an occupied block the MEM_TAG member is used.
*    - In case of a free block the payload area contains an additional
*      FREE_TAG structure which is linked within the list of free memory blocks.
*/
struct TAG {
  MEM_TAG  MemTag;
  FREE_TAG FreeTag;
};

/*********************************************************************
*
*       BLOCK
*/
typedef struct {
  U8 InUse; // Usage tag for memory block:
            //     BLOCK_FREE      - Block is free
            //     BLOCK_ALLOCATED - Block is allocated
            //   >=BLOCK_LOCKED    - Block is allocated and locked
  union {
    GUI_HMEM NextBlock; // If block is free, handle of next (free) block
    TAG *    pTag;      // If block is in use, pointer to memory tag
  } u;
} BLOCK;

/*********************************************************************
*
*       FREE_MEM
*/
typedef struct {
  U32        MinSize;       // Minimum size of memory blocks linked in the list
  FREE_TAG * pFirstFreeTag; // Pointer to first tag
} FREE_MEM;

/*********************************************************************
*
*       CONFIG
*/
typedef struct {
  void * pData;        // Pointer to assigned memory block
  U32    NumBytes;     // Size of memory block in bytes
  U32    AvBlockSize;  // Average block size, initial value without configuration is 32
} CONFIG;

/*********************************************************************
*
*       GUI_HEAP
*
*  Function description
*    32 bit aligned memory block to be managed by the memory management.
*/
typedef struct {
  U8 abHeap[1];
} GUI_HEAP;

/*********************************************************************
*
*       GUI_ALLOC_CONTEXT
*/
typedef struct {
  U32        NumFreeBytes;        // Number of free bytes
  U32        NumFreeBlocks;       // Number of free blocks
  U32        MaxUsedBytes;        // Peak of memory usage
  U32        AllocSize;           // Number of bytes available for memory management
  U32        NumBlocks;           // Number of available memory blocks
  U32        NumFixedBytes;       // Number of bytes in fixed blocks
  int        LockCnt;             // If >0 moving of memory is not allowed
  int        LockCntHandles;      // If >0 a pointer has been locked and moving memory is not allowed
  int        MaxRequestPercent;   // Percentage of memory allocatable by automatic memory devices
  GUI_HMEM   FirstFreeBlock;      // Index of first free block *
  U8         IsInitialized;       // 1 if already initialized
  U8         SuppressPeak;        // 1 if peaks should not be measured
  GUI_HMEM   hBlocks;             // Index of block which contains the BLOCK array *
  unsigned   MinFreeBlocks;

  BLOCK    * pBlocks;             // Pointer to block array within the assigned memory block
  GUI_HEAP * pHeap;               // Pointer to memory area within the assigned memory block
  FREE_TAG * pFirstFixedFreeTag;  // Linked list of free fixed memory blocks
  //
  // Array of linked lists with chains of free memory blocks.
  // It is sorted by potencies of 2. It is sorted by the minimum
  // of available memory. The blocks of the first list contain
  // at minimum <GUI_ALLOC_MIN_BLOCKSIZE> bytes of memory.
  // The memory blocks of the following lists contain at minimum
  //
  // <GUI_ALLOC_MIN_BLOCKSIZE> << 1
  // <GUI_ALLOC_MIN_BLOCKSIZE> << 2
  // <GUI_ALLOC_MIN_BLOCKSIZE> << 3
  // ...
  // <GUI_ALLOC_MIN_BLOCKSIZE> << 15
  //
  // bytes of memory. */
  //
  FREE_MEM   aFreeMem[16];
} GUI_ALLOC_CONTEXT;

extern GUI_ALLOC_CONTEXT GUI_ALLOC__Context; /* Memory allocation context, not static for debug purpose */
GUI_ALLOC_CONTEXT * const ptrGC = &GUI_ALLOC__Context;
/************************************************************** remove ***************************************/

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/**
 * @brief GUI buffers required for emwin library
 */
#define GUI_BUF_ADDR  0xA0000000
#define GUI_NUMBYTES  ((1024 * 1024) * 2)
const U32 GUI_Memory_Size = GUI_NUMBYTES;
const U32 GUI_Block_Size = 128;
U32 *const GUI_Memory = (U32 *) GUI_BUF_ADDR;

/**
 * Systick 1mS clock required for emWin time functions
 */
volatile uint32_t systick_timems;

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* mSec delay */
static void lcdDelay(uint32_t delay)
{
	delay += systick_timems;
	while (systick_timems < delay) {}
}

/* Initialize the LCD for the current board */
static void lcdInit(void)
{
	/* Board specific LCD pre-setup */
	Board_LCD_Init();

	/* Setup for current board */
	Chip_LCD_Init(LPC_LCD, (LCD_CONFIG_T *) &BOARD_LCD);
	Chip_LCD_SetUPFrameBuffer(LPC_LCD, (void *) FRAMEBUFFER_ADDR);
	Chip_LCD_PowerOn(LPC_LCD);
	lcdDelay(1000);

	/* Turn on backlight */
	//Board_SetLCDBacklight(1);
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

void * emWin_memcpy(void *dest, const void *src, long size)
{
	return memcpy(dest, src, size);
}
/**
 * @brief	Systick handler
 * @return	Nothing
 */
void SysTick_Handler(void)
{
	systick_timems++;
}



/**
 * @brief	Main routine for emWni Hello World example
 * @return	Nothing
 */
int main(void)
{
	int xPos, yPos, xSize;
	int i = 0;

	SystemCoreClockUpdate();
	Board_Init();

	/* sysTick will handle touch events at 1KHz */
	SysTick_Config(SystemCoreClock / 1000);

	/* Setup LCD */
	lcdInit();

	/* emWin start */
	GUI_Init();

	/* Solid color display */
	GUI_SetBkColor(GUI_RED);
	GUI_Clear();
	GUI_Delay(1000);
	GUI_SetBkColor(GUI_GREEN);
	GUI_Clear();
	GUI_Delay(1000);
	GUI_SetBkColor(GUI_BLUE);
	GUI_Clear();
	GUI_Delay(1000);
	GUI_SetBkColor(GUI_BLACK);
	GUI_Clear();

	xPos = LCD_GetXSize() / 2;
	yPos = LCD_GetYSize() / 3;
	GUI_SetColor(GUI_BROWN);
	GUI_SetTextMode(GUI_TM_REV);
	GUI_SetFont(GUI_FONT_20F_ASCII);
	GUI_DispStringHCenterAt("Hello NXP", xPos, yPos);
	GUI_SetFont(GUI_FONT_D24X32);
	GUI_SetColor(GUI_LIGHTYELLOW);
	xSize = GUI_GetStringDistX("0000");
	xPos -= xSize / 2;
	yPos += 24 + 10;
	while (1) {
		GUI_DispDecAt(i++, xPos, yPos, 4);
		if (i > 9999) {
			i = 0;
		}

		GUI_Delay(10);
	}
}
