/*
 * @brief Clock output example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2014
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

/* Select clock and divider */
#define CLKOUT_SEL      SYSCON_CLKOUTSRC_MAINCLK
#define CLKOUT_DIV      10

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

static AUDPLL_SETUP_T pll_setup_48KHz = {
	0x00007C40,
	0x0000002D,
	0x0000001B,
	0x0002074,
	0x00800000,
	24576000,
	PLL_SETUPFLAG_WAITLOCK,				/*!< PLL setup flags, Or'ed value of PLL_SETUPFLAG_* definitions */
};

/*****************************************************************************
 * Public functions
 ****************************************************************************/
struct clk_list {
	CHIP_SYSCON_CLKOUTSRC_T src;
	const char *name;
} clk_list[] = {
	{SYSCON_CLKOUTSRC_MAINCLK, "Main Clock"},
	{SYSCON_CLKOUTSRC_WDTOSC,  "Watchdog Osc"},
	{SYSCON_CLKOUTSRC_FROHF, 	"FRO HF Osc"},
	{SYSCON_CLKOUTSRC_RTC, 		"RTC Osc"},
	{SYSCON_CLKOUTSRC_SYSPLL,   "Sys PLL"},
	{SYSCON_CLKOUTSRC_AUDIOPLL, "Audio PLL"}
};

/* C Entry point */
int main(void)
{
	int loop = 1;	/* Prevents the unreachable statement warning */
	int count = (sizeof(clk_list) / sizeof(clk_list[0]));
	int i = 0;

	PLL_CONFIG_T pllConfig;
	PLL_SETUP_T pllSetup;
	PLL_ERROR_T pllError;

	/* Board Initialization */
	SystemCoreClockUpdate();
	Board_Init();

	/* Clock enables and checks */
	DEBUGSTR("Press any key to cycle through clocks...\r\n");

	Chip_Clock_EnablePeriphClock(SYSCON_CLOCK_RTC);
	/* Turn on the RTC 32K Oscillator */
	Chip_RTC_PowerUp(LPC_RTC);
	Chip_Clock_EnableRTCOsc();
	Chip_Clock_DisablePeriphClock(SYSCON_CLOCK_RTC);

	/* Enable the power to the WDT Oscillator */
	Chip_SYSCON_PowerUp(SYSCON_PDRUNCFG_PD_WDT_OSC);

	Chip_Clock_SetAudioPLLSource(SYSCON_AUDPLLCLKSRC_FRO12MHZ);
	Chip_Clock_SetAudioPLLFreq(&pll_setup_48KHz);

	/* Select the PLL input to the IRC */
	Chip_Clock_SetSystemPLLSource(SYSCON_PLLCLKSRC_FRO12MHZ);
	/* Power down PLL to change the PLL divider ratio */
	Chip_SYSCON_PowerDown(SYSCON_PDRUNCFG_PD_SYS_PLL);
	/* Setup PLL configuration */
	pllConfig.desiredRate = 72000000;
	pllConfig.InputRate = 0;
	pllConfig.flags = PLL_CONFIGFLAG_FORCENOFRACT;
	pllError = Chip_Clock_SetupPLLData(&pllConfig, &pllSetup);
	if (pllError == PLL_ERROR_SUCCESS) {
		pllSetup.flags = PLL_SETUPFLAG_WAITLOCK;
		pllError = Chip_Clock_SetupSystemPLLPrec(&pllSetup);
	}

	/* Map P3.12 as CLKOUT pin */
	Chip_IOCON_PinMuxSet(LPC_IOCON, 3, 12,
						 (IOCON_FUNC5 | IOCON_MODE_INACT | IOCON_DIGITAL_EN));

	while (loop) {
		if (i >= count) {
			i = 0;
		}
		DEBUGOUT("Clock selected : %s\r\n", clk_list[i].name);
		Chip_Clock_SetCLKOUTSource(clk_list[i].src, CLKOUT_DIV);
		i++;
		while (Board_UARTGetChar() == EOF) {}
	}
	return 0;
}
