/*
 * @brief	Simple ethernet example
 *			This simple MAC example will listen for all packets on
 *			the ethernet and display some stats via UART when a
 *			packet is received. A dummy packet can also be sent out.
 *			Wireshark can be used to view the outgoing packet.
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"
#include <stdio.h>
#include <string.h>

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

#define ENET_NUM_TX_DESC 8
#define ENET_NUM_RX_DESC 8

/*Should be word aligned */
ALIGN(16) static ENET_TXDESC_RD_T TXDescs[ENET_NUM_TX_DESC];
ALIGN(16) static ENET_RXDESC_RD_T RXDescs[ENET_NUM_RX_DESC];

/* Transmit/receive buffers and indices */
ALIGN(16) static uint8_t TXBuffer[ENET_NUM_TX_DESC][EMAC_ETH_MAX_FLEN];
ALIGN(16) static uint8_t RXBuffer[ENET_NUM_RX_DESC][EMAC_ETH_MAX_FLEN];
static int32_t rxFill, rxGet, rxAvail, rxNumDescs;
static int32_t txFill, txGet, txUsed, txNumDescs;

static uint8_t MACAddr[6] = {0x56, 0x34, 0x12, 0x37, 0x60, 0x00};

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* Local index and check function */
static __INLINE int32_t incIndex(int32_t index, int32_t max)
{
	index++;
	if (index >= max) {
		index = 0;
	}

	return index;
}

/* Initialize MAC descriptors for simple packet receive/transmit */
static void InitDescriptors(
	ENET_TXDESC_RD_T *pTXDescs, int32_t numTXDescs,
	ENET_RXDESC_RD_T *pRXDescs, int32_t numRXDescs)
{
	int i;

	/* Setup the descriptor list to a default state */
	memset(pTXDescs, 0, numTXDescs * sizeof(*pTXDescs));
	memset(pTXDescs, 0, numRXDescs * sizeof(*pRXDescs));
	rxFill = rxGet = 0;
	rxAvail = rxNumDescs = numRXDescs;
	txNumDescs = numTXDescs;
	txUsed = txGet = txFill = 0;

	/* Build linked list, CPU is owner of descriptors */
	for (i = 0; i < numTXDescs; i++) {
		pTXDescs[i].BSIZE = 0;
		pTXDescs[i].CTRLSTAT = 0;
		pTXDescs[i].B2ADD = 0;
	}
	for (i = 0; i < numRXDescs; i++) {
		pRXDescs[i].B2ADD = 0;
		pRXDescs[i].CTRL = 0;
	}

	/* Setup list pointers in Ethernet controller */
	Chip_ENET_SetDescRingLength(LPC_ETHERNET, 0, 7, 7);
	Chip_ENET_InitDescriptors(LPC_ETHERNET, 0, &pTXDescs[0], &pTXDescs[0], &pRXDescs[0], &pRXDescs[numRXDescs]);
}

/* Attach a buffer to a descriptor and queue it for reception */
static void ENET_RXQueue(uint8_t channel, void *buffer, int32_t bytes)
{
	if (rxAvail > 0) {
		/* Queue the next descriptor and start polling */
		RXDescs[rxFill].B1ADD = (uint32_t) buffer;
		if(rxFill == (rxNumDescs - 1)) {
			RXDescs[rxFill].B2ADD = (uint32_t)&RXDescs[0];
		}
		else {
			RXDescs[rxFill].B2ADD = 0;
		}
		RXDescs[rxFill].CTRL = RDES_NORM_RD_BUF1V | RDES_NORM_RD_OWN;
		rxAvail--;
		rxFill = incIndex(rxFill, rxNumDescs);
	}
}

/* Returns a pointer to a filled ethernet buffer or NULL if none are available */
static void *ENET_RXGet(int32_t *bytes)
{
	void *buffer;
	uint8_t i;

	/* This doesn't check status of the received packet */
	if ((rxAvail < rxNumDescs) && (!(RXDescs[rxGet].CTRL & RDES_NORM_WR_OWN))) {
		/* CPU owns descriptor, so a packet was received */
		if(RXDescs[rxGet].CTRL & RDES_NORM_WR_LD) {
			buffer = (void *) RXBuffer[rxGet];
			*bytes = (int32_t) RXDescs[rxGet].CTRL & RDES_NORM_WR_LENMASK;
		}
		else {
			/* Nothing received */
			*bytes = 0;
			buffer = NULL;
		}
		rxGet = incIndex(rxGet, rxNumDescs);
		rxAvail++;
	}
	else {
		/* Nothing received */
		*bytes = 0;
		buffer = NULL;
		if(Chip_ENET_ChannelStatus(LPC_ETHERNET, 0) & DMA_ST_RU) {
			/* Restart DMA Rx on error */
			Chip_ENET_StopDMARx(LPC_ETHERNET, 0);
			for(i = 0; i < 200; i++);
			Chip_ENET_StartDMARx(LPC_ETHERNET, 0);
		}
	}

	return buffer;
}

/* Attaches a buffer to a transmit descriptor and queues it for transmit */
static void ENET_TXQueue(uint8_t channel, void *buffer, int32_t bytes)
{
	if (txUsed < txNumDescs) {
		/* Queue the next descriptor and start polling */
		TXDescs[txFill].B1ADD = (uint32_t) buffer;
		TXDescs[txFill].B2ADD = 0;
		TXDescs[txFill].BSIZE = TDES_NORM_RD_BS1(bytes);
		TXDescs[txFill].CTRLSTAT = TDES_NORM_RD_FL(bytes) | TDES_NORM_RD_SAIC(2) | TDES_NORM_RD_FD | TDES_NORM_RD_LD | TDES_NORM_RD_OWN;

		LPC_ETHERNET->DMA_CHANNEL[channel].DMA_CHx_TXDESC_TAIL_PTR  = ((uint32_t) &TXDescs[txFill+1]) & ~0x03;
		txUsed++;
		txFill = incIndex(txFill, txNumDescs);
		/* Start DMA */
		Chip_ENET_StartDMATx(LPC_ETHERNET, channel);
		Chip_ENET_TXEnable(LPC_ETHERNET);
	}
}

/* Returns a pointer to a buffer that has been transmitted */
static void *ENET_TXBuffClaim(void)
{
	void *buffer;

	/* Is packet done sending? */
	if ((txUsed > 0) && (!(TXDescs[txGet].CTRLSTAT & TDES_NORM_RD_OWN))) {
		/* CPU owns descriptor, so the packet completed transmit */
		buffer = (void *) TXBuffer[txGet];
		txGet = incIndex(txGet, txNumDescs);
		txUsed--;
		Board_LED_Set(2, false);
		Chip_ENET_TXDisable(LPC_ETHERNET);
		Chip_ENET_StopDMATx(LPC_ETHERNET, 0);
	}
	else {
		buffer = NULL;
	}

	return buffer;
}

static void phy_write(uint8_t reg, uint16_t data)
{
	Chip_ENET_StartMIIWrite(LPC_ETHERNET, reg, data);
	while(Chip_ENET_IsMIIBusy(LPC_ETHERNET));
}

static uint16_t phy_read(uint8_t reg)
{
	Chip_ENET_StartMIIRead(LPC_ETHERNET, reg);
	while(Chip_ENET_IsMIIBusy(LPC_ETHERNET));
	return Chip_ENET_ReadMIIData(LPC_ETHERNET);
}
static void phy_init()
{
	uint16_t temp;
	/*Reset and wait until reset completion */
	phy_write(0, (uint16_t)0x8000);
	do {
		temp = phy_read(0);
	}	while(temp & 0x8000);

	/* Start Auto negotiation and wait until auto negotiation completion */
	phy_write(0, (uint16_t)(0x1200));
	do {
		temp = phy_read(0x1f);
	}	while((temp & 0x1000) == 0);

	/* Based on the phy status set speed and duplex mode */
	switch((temp >> 2) & 0x07) {
	case 0x01:
		/* Half Duplex and 10Mbps */
		Chip_ENET_SetDuplex(LPC_ETHERNET, (bool) 0);
		Chip_ENET_SetSpeed(LPC_ETHERNET, (bool) 0);
		break;
	case 0x05:
		/* Full Duplex and 10Mbps */
		Chip_ENET_SetDuplex(LPC_ETHERNET, (bool) 1);
		Chip_ENET_SetSpeed(LPC_ETHERNET, (bool) 0);
		break;
	case 0x02:
		/* Half Duplex and 100Mbps */
		Chip_ENET_SetDuplex(LPC_ETHERNET, (bool) 0);
		Chip_ENET_SetSpeed(LPC_ETHERNET, (bool) 1);
		break;
	case 0x06:
		/* Full Duplex and 100Mbps */
		Chip_ENET_SetDuplex(LPC_ETHERNET, (bool) 1);
		Chip_ENET_SetSpeed(LPC_ETHERNET, (bool) 1);
		break;
	}
}

static uint16_t phy_status()
{
	return phy_read(0x1);
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	Handle interrupt from ethernet
 * @return	Nothing
 */
void ETH_IRQHandler(void)
{
	/* This demo is entirely polled, so the IRQ isn't really needed */
}

/**
 * @brief	main routine for ENET example
 * @return	Nothing (function should not exit)
 */
int main(void)
{
	uint8_t *workbuff;
	int32_t rxBytes, i, txNextIndex;
	uint32_t physts = 0;
	bool ethpkttgl = true;

	/* LED0 is used for the link status, on = PHY cable detected */
	SystemCoreClockUpdate();
	Board_Init();
	Board_Ethernet_Init();

	/* Initial LED state is off to show an unconnected cable state */
	Board_LED_Set(0, false);

	Chip_ENET_RMIIEnable(LPC_ETHERNET);

	/* Setup ethernet and PHY */
	Chip_ENET_Init(LPC_ETHERNET, 0);
	phy_init();

	/* Setup MAC address for device */
	Chip_ENET_SetADDR(LPC_ETHERNET, 0, 0, MACAddr);

	Chip_ENET_SetTxBurstLen(LPC_ETHERNET, 0, 8, 0);
	Chip_ENET_SetRxBurstLen(LPC_ETHERNET, 0, 8, 0);

	/* Setup descriptors */
	InitDescriptors(TXDescs, ENET_NUM_TX_DESC, RXDescs, ENET_NUM_RX_DESC);

	/* Attach a buffer to a RX descriptor and queue it for receive */
	i = 0;
	while (i < ENET_NUM_RX_DESC) {
		ENET_RXQueue(0, RXBuffer[i], EMAC_ETH_MAX_FLEN);
		i++;
	}
	txNextIndex = 0;

	Chip_ENET_TXEnable(LPC_ETHERNET);
	/* Enable RX/TX after descriptors are setup */
	Chip_ENET_RXEnable(LPC_ETHERNET);
	Chip_ENET_StartDMARx(LPC_ETHERNET, 0);

	DEBUGOUT("Start of Ethernet Example: \r\n");
	while (1) {
		/* Check for receive packets */
		workbuff = ENET_RXGet(&rxBytes);
		if (workbuff) {
			/* Packet was received. Dump some info from the packet */
			DEBUGOUT("Packet received: ");
			ethpkttgl = (bool) !ethpkttgl;

			/* Toggle LED2 on each packet received */
			Board_LED_Set(1, ethpkttgl);

			/* Destination and source MAC addresses */
			DEBUGOUT("-Dest MAC: %02x:%02x:%02x:%02x:%02x:%02x-",
					 workbuff[0], workbuff[1], workbuff[2], workbuff[3], workbuff[4], workbuff[5]);
			DEBUGOUT("-Source MAC: %02x:%02x:%02x:%02x:%02x:%02x-",
					 workbuff[6], workbuff[7], workbuff[8], workbuff[9], workbuff[10], workbuff[11]);

			/* Length of received packet and embedded len/type */
			DEBUGOUT("-Packet len: %d", rxBytes);
			DEBUGOUT("-Type: %04x\r\n", ((((uint32_t) workbuff[12]) << 8) | (((uint32_t) workbuff[13]))));

			/* Re-queue the (same) packet again */
			ENET_RXQueue(0, workbuff, EMAC_ETH_MAX_FLEN);
		}

		/* Send a 'dummy' broadcast packet on a keypress. You'll only see this
		     if your using a tool such as WireShark and the packet will not have
		   a checksum */
		if (DEBUGIN() != EOF) {
			/* Only if link detected */
			if (physts & 0x04) {
				/* Get next available TX buffer */
				workbuff = (uint8_t *) TXBuffer[txNextIndex];
				txNextIndex = incIndex(txNextIndex, ENET_NUM_TX_DESC);
				/* Destination is broadcast */
				for (i = 0; i < 6; i++) {
					workbuff[i] = 0xFF;
				}

				/* Source is this MAC address */
				memcpy(&workbuff[6], MACAddr, 6);

				/* Size will be 128 bytes (total) */
				workbuff[12] = 0;
				workbuff[13] = 128;

				/* Some dummy data, fill beyond end of packet */
				for (i = 0; i < 128; i++) {
					workbuff[i + 14] = (uint8_t) (i & 0xFF);
				}

				/* Queue TX packet. This is an Ethernet packet, but the payload and
				   hecksum are garbage. The packet will still be viewable with
				   Wireshark */
				ENET_TXQueue(0, workbuff, 142); /* Payload size 128 + 14 bytes of header */
				Board_LED_Set(2, true);
			}
		}

		/* Transmit buffers are 'zero-copy' buffers with the queue function, so
		   the buffer must remain in memory until the packet has been fully
		   transmitted. Call ENET_TXBuffClaim() to to reclaim a sent
		   packet. If a packet buffer address is return (not NULL), then the
		   packet can be de-allocated. Since the buffers in this examples are
		   static, there isn't too much to do. */
		ENET_TXBuffClaim();

		/* PHY status state machine, LED on when connected. This function will
		   not block. */
		physts = phy_status();

		/* Only check for connection state when the PHY status has changed */
		if (physts & 0x04) {
			Board_LED_Set(0, true);
		}
		else {
			Board_LED_Set(0, false);
		}
	}

	/* Never returns, for warning only */
	return 0;
}
