/*
 * @brief This file contains a USB MSC RAM example using USB ROM Drivers.
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"
#include <stdio.h>
#include <string.h>
#include "app_usbd_cfg.h"
#include "msc_disk.h"

#if USE_USB_HIGH_SPEED
#define LPC_USB							LPC_USB1
#define LPC_USB_BASE				LPC_USB1_BASE
#define USB_IRQn						USB1_IRQn
#define USB_IRQHandler			USB1_IRQHandler
#else
#define LPC_USB							LPC_USB0
#define LPC_USB_BASE				LPC_USB0_BASE
#define USB_IRQn						USB0_IRQn
#define USB_IRQHandler			USB0_IRQHandler
#endif

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/
static USBD_HANDLE_T g_hUsb;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/
const USBD_API_T *g_pUsbApi;

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/* Interrupt service routine of USB */
void USB_IRQHandler(void)
{
	USBD_API->hw->ISR(g_hUsb);
}


/* Function to find the interface descriptor */
USB_INTERFACE_DESCRIPTOR *find_IntfDesc(const uint8_t *pDesc, uint32_t intfClass)
{
	USB_COMMON_DESCRIPTOR *pD;
	USB_INTERFACE_DESCRIPTOR *pIntfDesc = 0;
	uint32_t next_desc_adr;

	pD = (USB_COMMON_DESCRIPTOR *) pDesc;
	next_desc_adr = (uint32_t) pDesc;

	while (pD->bLength) {
		/* is it interface descriptor */
		if (pD->bDescriptorType == USB_INTERFACE_DESCRIPTOR_TYPE) {

			pIntfDesc = (USB_INTERFACE_DESCRIPTOR *) pD;
			/* did we find the right interface descriptor */
			if (pIntfDesc->bInterfaceClass == intfClass) {
				break;
			}
		}
		pIntfDesc = 0;
		next_desc_adr = (uint32_t) pD + pD->bLength;
		pD = (USB_COMMON_DESCRIPTOR *) next_desc_adr;
	}

	return pIntfDesc;
}

/* Main application entry point */
int main(void)
{
	int loop = 1;	/* Used to fix the unreachable statement warning */

	extern void DataRam_Initialize(void);
	USBD_API_INIT_PARAM_T usb_param;
	USB_CORE_DESCS_T desc;
	ErrorCode_t ret = LPC_OK;

	/* Initialize board and chip */
	SystemCoreClockUpdate();
	Board_Init();

	/* enable clocks and pinmux */
	/* USB clock was set in board_init(0 already. enable USB peripheral clock here only */
#if USE_USB_HIGH_SPEED
	Chip_USB1_Init();
	Chip_Clock_EnablePeriphClock(SYSCON_CLOCK_USB1_HOST);
	*(uint32_t *)(LPC_USB1_HOST_BASE + 0x50) |= (1 << 16);
	Chip_Clock_DisablePeriphClock(SYSCON_CLOCK_USB1_HOST);
#else
	Chip_USB0_Init();
	/* Enable USB Host Slave Interface Peripheral clock */
	Chip_Clock_EnablePeriphClock(SYSCON_CLOCK_USB0_HOST_S);
	/* Enable Device mode */
	*(uint32_t *)(LPC_USB0_HOST_BASE + 0x5C) |= (1 << 16);
	/* Disable USB Host Slave Interface Peripheral clock */
	Chip_Clock_DisablePeriphClock(SYSCON_CLOCK_USB0_HOST_S);
#endif
	
	DataRam_Initialize();

	/* Init USB API structure */
	g_pUsbApi = (const USBD_API_T *) LPC_ROM_API->usbdApiBase;

	/* initialize call back structures */
	memset((void *) &usb_param, 0, sizeof(USBD_API_INIT_PARAM_T));
	usb_param.usb_reg_base = LPC_USB_BASE;
	usb_param.mem_base = USB_STACK_MEM_BASE;
	usb_param.mem_size = USB_STACK_MEM_SIZE;
	usb_param.max_num_ep = 2;

	/* Set the USB descriptors */
	desc.device_desc = (uint8_t *) USB_DeviceDescriptor;
	desc.string_desc = (uint8_t *) USB_StringDescriptor;

	/* Note, to pass USBCV test full-speed only devices should have both
	   descriptor arrays point to same location and device_qualifier set to 0.
	 */
#if USE_USB_HIGH_SPEED
	usb_param.high_speed_capable = TRUE;
	desc.high_speed_desc = USB_HsConfigDescriptor;
	desc.full_speed_desc = USB_FsConfigDescriptor;
	desc.device_qualifier = (uint8_t *)USB_DeviceQualifier;
#else
	usb_param.high_speed_capable = FALSE;
	desc.high_speed_desc = USB_FsConfigDescriptor;
	desc.full_speed_desc = USB_FsConfigDescriptor;
	desc.device_qualifier = 0;
#endif

	/* USB Initialization */
	ret = USBD_API->hw->Init(&g_hUsb, &desc, &usb_param);
	if (ret == LPC_OK) {
#if USE_USB_HIGH_SPEED
		ret = mscDisk_init(g_hUsb, (uint8_t *)&USB_HsConfigDescriptor[sizeof(USB_CONFIGURATION_DESCRIPTOR)], &usb_param);
#else
		ret = mscDisk_init(g_hUsb, (uint8_t *)&USB_FsConfigDescriptor[sizeof(USB_CONFIGURATION_DESCRIPTOR)], &usb_param);
#endif
		if (ret == LPC_OK) {
			/*  enable USB interrrupts */
			NVIC_EnableIRQ(USB_IRQn);
			/* now connect */
			USBD_API->hw->Connect(g_hUsb, 1);
		}
	}

	while (loop) {
		/* Sleep until next IRQ happens */
		__WFI();
	}
	return 0;
}
