/*
 * @brief HID generic example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"
#include <stdio.h>
#include <string.h>
#include "app_usbd_cfg.h"
#include "hid_generic.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

#if USE_USB_HIGH_SPEED
#define LPC_USB							LPC_USB1
#define LPC_USB_BASE				LPC_USB1_BASE
#define USB_IRQn						USB1_IRQn
#define USB_IRQHandler			USB1_IRQHandler
#else
#define LPC_USB							LPC_USB0
#define LPC_USB_BASE				LPC_USB0_BASE
#define USB_IRQn						USB0_IRQn
#define USB_IRQHandler			USB0_IRQHandler
#endif

static USBD_HANDLE_T g_hUsb;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

const  USBD_API_T *g_pUsbApi;

#ifndef TIME_INTERVAL
#define TIME_INTERVAL	(SystemCoreClock/1000 - 1)
#endif

volatile uint32_t SuspendFlag = 0;
volatile uint32_t timer0_counter = 0;
volatile uint32_t USBWakeupInterruptFlag = 0;
volatile uint32_t WakeupFlag = 0;

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/*****************************************************************************
 * Public functions
 ****************************************************************************/
/*
 *  USB Suspend Function
 *   Called automatically on USB Suspend
 *    Parameters:      USBD Handle
 *    Return Value:    Error code
 */

ErrorCode_t USB_Suspend (USBD_HANDLE_T hUsb) {
  /* Performed by Hardware, Start the timer when SUSPEND interrupt occurs. */
  timer0_counter = 0;
  Chip_TIMER_Enable(LPC_TIMER0);
  
  if ( SuspendFlag == 0 ) {
    /* State change from ACTIVE to SUSPEND. */
    SuspendFlag = 1;
  }
	return (LPC_OK);
}


/*
 *  USB Resume Function
 *   Called automatically on USB Resume
 *    Parameters:      USBD Handle
 *    Return Value:    Error code
 */

ErrorCode_t USB_Resume (USBD_HANDLE_T hUsb) {
  /* Performed by Hardware, Sttop the timer when SUSPEND interrupt occurs. */
  Chip_TIMER_Disable(LPC_TIMER0);
  timer0_counter = 0;
  if ( SuspendFlag == 1 ) {
    /* State change from SUSPEND to RESUME or ACTIVE. */
    SuspendFlag = 0;
  }
	return (LPC_OK);
}

/******************************************************************************
** Function name:		CT32B0_IRQHandler
**
** Descriptions:		Timer/CounterX and captureX interrupt handler
**                  (Only match 0 is used in this example. )
**
** parameters:			None
** Returned value:	None
** 
******************************************************************************/
void CT32B0_IRQHandler(void)
{
  if ( Chip_TIMER_MatchPending(LPC_TIMER0, 0) ) {  
    Chip_TIMER_ClearMatch(LPC_TIMER0, 0);			/* clear interrupt flag */
    timer0_counter++;
  }
  return;
}

/**
 * @brief	Handle interrupt from USB
 * @return	Nothing
 */
void USB_IRQHandler(void)
{
	USBD_API->hw->ISR(g_hUsb);
}

/**
 * @brief	USB Wakeup interrupt
 * @return	Nothing
 */
#if USE_USB_HIGH_SPEED
void USB1_EnableDisablePHY( int32_t enable_flag )
{
	uint32_t *otg_port_mode;

	Chip_Clock_EnablePeriphClock(SYSCON_CLOCK_USB1_HOST);	
	otg_port_mode = (uint32_t *)(LPC_USB1_HOST_BASE + 0x50);
	*otg_port_mode |= (0x1 << 18);			/* Enable S/W control */
	if ( enable_flag ) {

		*otg_port_mode &= ~(0x1 << 19);		/* PHY operational */
	}
	else {
		*otg_port_mode |= (0x1 << 19);
	}
	Chip_Clock_DisablePeriphClock(SYSCON_CLOCK_USB1_HOST);
}

void USB1_NEEDCLK_IRQHandler(void)
{
  /* Below clear pending is not needed as NeedClk can be configured
  as rising or falling edge, not level trigger anymore. */  
//  NVIC_ClearPendingIRQ(USBNeedClk_IRQn);   /* If pending, clear USB Activity interrupt. */ 
  NVIC_DisableIRQ(USB1ACT_IRQn);        /* Disable USB activity interrupt */
  if ( WakeupFlag == 1 ) {
		/* State change from low-power mode to wake-up. */
    WakeupFlag = 0;
    USBWakeupInterruptFlag = 1;
  }
  return;
}
#else
void USB0_NEEDCLK_IRQHandler(void)
{
  /* Below clear pending is not needed as NeedClk can be configured
  as rising or falling edge, not level trigger anymore. */  
//  NVIC_ClearPendingIRQ(USBNeedClk_IRQn);   /* If pending, clear USB Activity interrupt. */ 
  NVIC_DisableIRQ(USB0ACT_IRQn);        /* Disable USB activity interrupt */
  if ( WakeupFlag == 1 ) {
		/* State change from low-power mode to wake-up. */
    WakeupFlag = 0;
    USBWakeupInterruptFlag = 1;
  }
  return;
}
#endif

/**
 * @brief	Find the address of interface descriptor for given class type.
 * @return	If found returns the address of requested interface else returns NULL.
 */
USB_INTERFACE_DESCRIPTOR *find_IntfDesc(const uint8_t *pDesc, uint32_t intfClass)
{
	USB_COMMON_DESCRIPTOR *pD;
	USB_INTERFACE_DESCRIPTOR *pIntfDesc = 0;
	uint32_t next_desc_adr;

	pD = (USB_COMMON_DESCRIPTOR *) pDesc;
	next_desc_adr = (uint32_t) pDesc;

	while (pD->bLength) {
		/* is it interface descriptor */
		if (pD->bDescriptorType == USB_INTERFACE_DESCRIPTOR_TYPE) {

			pIntfDesc = (USB_INTERFACE_DESCRIPTOR *) pD;
			/* did we find the right interface descriptor */
			if (pIntfDesc->bInterfaceClass == intfClass) {
				break;
			}
		}
		pIntfDesc = 0;
		next_desc_adr = (uint32_t) pD + pD->bLength;
		pD = (USB_COMMON_DESCRIPTOR *) next_desc_adr;
	}

	return pIntfDesc;
}

/**
 * @brief	main routine for USB device example
 * @return	Function should not exit.
 */
int main(void)
{
	USBD_API_INIT_PARAM_T usb_param;
	USB_CORE_DESCS_T desc;
	ErrorCode_t ret = LPC_OK;

	SystemCoreClockUpdate();
	/* Initialize board and chip */
	Board_Init();

	/* enable USB clocks and pinmux */
#if USE_USB_HIGH_SPEED
	Chip_USB1_Init();
	LPC_USB->DEVCMDSTAT &= ~USB_PLL_ON;	/* NeedClk normal mode(not ALWAYS ON) */
	LPC_SYSCON->USB1CLKCTRL = (0x1<<1) | (0x1<<4);  /* USB wakeup interrupt on rising edge of Needclk. */
	
	/* In OTG PORT MODE register of the USB host controller, the USB port is routed to USB host on power_on Reset, 
	   in order to switch to device, enable the clock to the host controller and route the USB port to the USB 
		 device first, then diasble the clock to the host to save power. */
	Chip_Clock_EnablePeriphClock(SYSCON_CLOCK_USB1_HOST);
	/* REMARK: For USB HS device application using wakeup interrupt, the host needclk must be going away first
	before setiing DEV_ENABLE bit in PORT MODE register as seen below!!! */
	while ( (LPC_SYSCON->USB1CLKSTAT & 0x02) );
	*(uint32_t *)(LPC_USB1_HOST_BASE + 0x50) |= (0x1 << 16);
	Chip_Clock_DisablePeriphClock(SYSCON_CLOCK_USB1_HOST);
#else
	Chip_USB0_Init();
	LPC_USB->DEVCMDSTAT &= ~USB_PLL_ON;	/* NeedClk normal mode(not ALWAYS ON) */
	LPC_SYSCON->USB0CLKCTRL = 0x1<<1;  /* USB wakeup interrupt on rising edge of Needclk. */
	/* In OTG PORT MODE register of the USB host controller, the USB port is routed to USB host on power_on Reset, 
	   in order to switch to device, enable the clock to the host controller and route the USB port to the USB 
		 device first, then diasble the clock to the host to save power. */
	Chip_Clock_EnablePeriphClock(SYSCON_CLOCK_USB0_HOST_S);
	*(uint32_t *)(LPC_USB0_HOST_BASE + 0x5C) |= (0x1 << 16);
	Chip_Clock_DisablePeriphClock(SYSCON_CLOCK_USB0_HOST_S);
#endif
	
	Chip_TIMER_Init(LPC_TIMER0);
	Chip_TIMER_SetMatch(LPC_TIMER0, 0, TIME_INTERVAL);
	Chip_TIMER_MatchEnableInt(LPC_TIMER0, 0);
	Chip_TIMER_ResetOnMatchEnable(LPC_TIMER0, 0);
	NVIC_EnableIRQ(CT32B0_IRQn);
	
	/* initialize USBD ROM API pointer. */
	g_pUsbApi = (const USBD_API_T *) LPC_ROM_API->usbdApiBase;
	
	/* initialize call back structures */
	memset((void *) &usb_param, 0, sizeof(USBD_API_INIT_PARAM_T));
	usb_param.usb_reg_base = LPC_USB_BASE;
	usb_param.max_num_ep = 2;
	usb_param.mem_base = USB_STACK_MEM_BASE;
	usb_param.mem_size = USB_STACK_MEM_SIZE;
	usb_param.USB_Suspend_Event = USB_Suspend;
	usb_param.USB_Resume_Event = USB_Resume;

	/* Set the USB descriptors */
	desc.device_desc = (uint8_t *) &USB_DeviceDescriptor[0];
	desc.string_desc = (uint8_t *) &USB_StringDescriptor[0];

	/* Note, to pass USBCV test full-speed only devices should have both
	 * descriptor arrays point to same location and device_qualifier set
	 * to 0.
	 */
#if USE_USB_HIGH_SPEED
	usb_param.high_speed_capable = TRUE;
	desc.high_speed_desc = USB_HsConfigDescriptor;
	desc.full_speed_desc = USB_FsConfigDescriptor;
	desc.device_qualifier = (uint8_t *)USB_DeviceQualifier;
#else
	usb_param.high_speed_capable = FALSE;
	desc.high_speed_desc = USB_FsConfigDescriptor;
	desc.full_speed_desc = USB_FsConfigDescriptor;
	desc.device_qualifier = 0;
#endif

	/* USB Initialization */
	ret = USBD_API->hw->Init(&g_hUsb, &desc, &usb_param);
	if (ret == LPC_OK) {
#if USE_USB_HIGH_SPEED
		ret = usb_hid_init(g_hUsb, (USB_INTERFACE_DESCRIPTOR *) &USB_HsConfigDescriptor[sizeof(USB_CONFIGURATION_DESCRIPTOR)],
						 &usb_param.mem_base, &usb_param.mem_size);
#else
		ret = usb_hid_init(g_hUsb, (USB_INTERFACE_DESCRIPTOR *) &USB_FsConfigDescriptor[sizeof(USB_CONFIGURATION_DESCRIPTOR)],
						 &usb_param.mem_base, &usb_param.mem_size);
#endif
		if (ret == LPC_OK) {
			/*  enable USB interrupts */
			NVIC_EnableIRQ(USB_IRQn);
			/* now connect */
			USBD_API->hw->Connect(g_hUsb, 1);
		}
	}

	while (1) {
		/* Sleep until next IRQ happens */
		if ( USBWakeupInterruptFlag ) {
			USBWakeupInterruptFlag = 0;
#if USE_USB_HIGH_SPEED
			USB1_EnableDisablePHY(1);			/* USB PHY needs to be reenabled. */
#endif
		}
		/* bit 8 on USN_INT_STAT is USB NeedClk bit. */
		if ( SuspendFlag == 1 ) {			
			/* If it's more than 5ms, it's not de-bounce but a true SUSPEND.
			Go to deep sleep or power down mode. */
			if ( timer0_counter > TIME_INTERVAL/20 ) {
				SuspendFlag = 0;
				timer0_counter = 0;
#if USE_USB_HIGH_SPEED
				while ( (LPC_SYSCON->USB1CLKSTAT & 0x01) );
				/* Enable USB activity as wakeup source. */
				/* Regular USB interrupt is disabled to test USBWakeup interrupt. 
				It will be enabled once it's waken up. */
				NVIC_DisableIRQ(USB1ACT_IRQn);
				NVIC_ClearPendingIRQ(USB1ACT_IRQn);
				NVIC_EnableIRQ(USB1ACT_IRQn);
				/* USB_Needclk in STARTERP logic domain. */
				Chip_SYSCON_EnableWakeup( SYSCON_STARTER_USB1NEEDCLK );
				WakeupFlag = 1;
				
				USB1_EnableDisablePHY(0);			/* Disable PHY */
				/* Before going to power down mode, SYSCLK should not be on PLL, otherwise, SYSCLK needs to be switched to
				FRO_12M or FRO_HF, and SYSPLL needs to be powered down. */
				if ( Chip_Clock_GetMainClockSource() == SYSCON_MAINCLKSRC_PLLOUT ) {
					/* Please note, in this test, USB1 clockd use USB PLL output, if SYSPLL is used as the USB clock,	
					USB PLL needd to be turned on and configured, then USB clock needs to be routed to USB PLL output before
					SYS PLL can be shut off. */
					Chip_Clock_SetMainClockSource(SYSCON_MAINCLKSRC_FROHF);
					Chip_SYSCON_PowerDown(SYSCON_PDRUNCFG_PD_SYS_PLL);
				}	
				/* Go to sleep leaving SRAM powered during sleep. Use lower
					voltage during sleep. */
				Chip_POWER_EnterPowerMode(POWER_POWER_DEEP, SYSCON_PDRUNCFG_PD_SRAM0|SYSCON_PDRUNCFG_PD_USB_RAM|SYSCON_PDRUNCFG_PD_VD5, 
								SYSCON_PDRUNCFG1_PD_USB1PHY);
				/* More peripherals need to be left on in order for USB HS to wake up. */
#else
				while ( (LPC_SYSCON->USB0CLKSTAT & 0x01) );
				/* Enable USB activity as wakeup source. */
				/* Regular USB interrupt is disabled to test USBWakeup interrupt. 
				It will be enabled once it's waken up. */
				NVIC_DisableIRQ(USB0ACT_IRQn);
				NVIC_ClearPendingIRQ(USB0ACT_IRQn);
				NVIC_EnableIRQ(USB0ACT_IRQn);
				/* USB_Needclk in STARTERP logic domain. */
				Chip_SYSCON_EnableWakeup( SYSCON_STARTER_USB0NEEDCLK );
				WakeupFlag = 1;
				/* Go to sleep leaving SRAM powered during sleep. Use lower
					voltage during sleep. */
				Chip_POWER_EnterPowerMode(POWER_POWER_DEEP, SYSCON_PDRUNCFG_PD_SRAMX|SYSCON_PDRUNCFG_PD_SRAM0, 0);
#endif
			}
		}
	}
}
